/*
    File:             consol_awf2.cpp
    Author:           Tomas Krejci, 29/10/2009, revised 03/03/2010
    Purpose:          material model for saturated-nonsaturated air and water flow in a deforming porous medium
    sources:          Lewis and Schrefler pp. 93-97, material parameters are set for benchmark on page n. 168
    unknowns:         number of unknowns=2, pw = liqud water pressure, pg = gas(air) pressure
*/ 

#include <stdio.h>
#include <stdlib.h>
#include <math.h>
#include "aliast.h"
#include "constrel.h"
#include "consol_awf2.h"
#include "globalt.h"

con_awf2mat::con_awf2mat()
{
  alpha = 1.0;//Biot's constant [-] alpha = 1 - kt/ks
  //
  ks = 2.167e6;//bulk modulus of solid phase (grains) [Pa]
  //
  kw = 2.0e9;//bulk modulus of water [Pa]
  //
  phi0 = 0.297;//initial porosity [-]
  //
  kintr = 4.5e-13;//intrinsic permeability [m^2]
  //
  rhow = 1000.0;//water density [kg/m^3]
  //
  muw0 = 1.0e-3;//water viscosity [Pa.s]
  //
  mug0 = 1.8e-5;//air viscosity [Pa.s]
  //
  p_atm = 101.325; //temporarily in kPa //101325.0; //atmospheric pressure [Pa]
}

con_awf2mat::~con_awf2mat()
{}


/**
   function reads parameters
   
   @param in - input file

   29/10/2009, TKr
*/
void con_awf2mat::read(XFILE *in)
{
  xfscanf (in,"%k%m","airwaterflowtype",&airwaterflowtype_kwdset, &model_type);
  switch (model_type){
  case lewis_and_schrefler2:{//Lewis and Schrefler's book
    xfscanf (in,"%lf %lf %lf %lf %lf %lf %lf %lf", &alpha, &ks, &phi0, &kw, &rhow, &muw0, &mug0, &kintr);
    break;
  }
  case lewis_and_schrefler2_mefel:{//Lewis and Schrefler's book
    xfscanf (in,"%lf %lf %lf %lf %lf %lf %lf %lf", &alpha, &ks, &phi0, &kw, &rhow, &muw0, &mug0, &kintr);
    break;
  }
  default:{
    fprintf (stderr,"\n unknown model type is required");
    fprintf (stderr,"\n in function (%s, line %d).\n",__FILE__,__LINE__);
  }
  } 
}


/**
   function reads parameters
   
   @param out - output file

   29/10/2009, TKr
*/
void con_awf2mat::print(FILE *out)
{
  fprintf (out,"  %lf %lf %lf %lf %lf \n",alpha,ks,phi0,kw,kintr); 
}




/**
   function computes conductivity matrix of the material
   in the required integration point
   
   @param d - conductivity %matrix of material
   @param ri  - row index
   @param ci  - column index   
   @param ipp - number of integration point

*/
void con_awf2mat::matcond (matrix &d,long ri,long ci,long ipp)
{
  long n;
  n = d.n;

  switch (n){
  case 1:{
    matcond1d (d,ri,ci,ipp);//1D
    break;
  }
  case 2:{
    matcond2d (d,ri,ci,ipp);//2D
    break;
  }
  case 3:{
    matcond3d (d,ri,ci,ipp);//3D
    break;
  }
  default:{
    fprintf (stderr,"\n unknown number of components of stress tensor is required");
    fprintf (stderr,"\n in function (%s, line %d).\n",__FILE__,__LINE__);
  }
  }
}


/**
   function creates conductivity matrix of the material for 1D problems

   @param d   - conductivity %matrix of the material
   @param ri  - row index
   @param ci  - column index   
   @param ipp - number of integration point
*/
void con_awf2mat::matcond1d (matrix &d,long ri,long ci,long ipp)
{
  double k;
  double pw,pg;
  k = 0.0;
  
  pw = Tm->ip[ipp].av[0];
  pg = Tm->ip[ipp].av[1];
  
  if((ri == 0) && (ci == 0))
    k = get_kww(pw,pg,ipp);
  if((ri == 0) && (ci == 1))
    k = get_kwg(pw,pg,ipp);
  if((ri == 1) && (ci == 0))
    k = get_kgw(pw,pg,ipp);
  if((ri == 1) && (ci == 1))
    k = get_kgg(pw,pg,ipp);
  
  d[0][0] = k;
}

/**
   function creates conductivity matrix of the material for 2D problems

   @param d   - conductivity %matrix of the material
   @param ri  - row index
   @param ci  - column index   
   @param ipp - number of integration point
*/
void con_awf2mat::matcond2d (matrix &d,long ri,long ci,long ipp)
{
  double k;
  double pw,pg;
  k = 0.0;
  
  pw = Tm->ip[ipp].av[0];
  pg = Tm->ip[ipp].av[1];
  
  if((ri == 0) && (ci == 0))
    k = get_kww(pw,pg,ipp);
  check_math_errel(0);      
  if((ri == 0) && (ci == 1))
    k = get_kwg(pw,pg,ipp);
  check_math_errel(0);      
  if((ri == 1) && (ci == 0))
    k = get_kgw(pw,pg,ipp);
  check_math_errel(0);      
  if((ri == 1) && (ci == 1))
    k = get_kgg(pw,pg,ipp);
  check_math_errel(0);      
  
  fillm(0.0,d);

  d[0][0] = k;   d[0][1] = 0.0;
  d[1][0] = 0.0; d[1][1] = k;
}

/**
   function creates conductivity matrix of the material for 3D problems

   @param d   - conductivity %matrix of the material
   @param ri  - row index
   @param ci  - column index   
   @param ipp - number of integration point
*/
void con_awf2mat::matcond3d (matrix &d,long ri,long ci,long ipp)
{
  double k;
  double pw,pg;
  k = 0.0;
  
  pw = Tm->ip[ipp].av[0];
  pg = Tm->ip[ipp].av[1];
  
  if((ri == 0) && (ci == 0))
    k = get_kww(pw,pg,ipp);
  if((ri == 0) && (ci == 1))
    k = get_kwg(pw,pg,ipp);
  if((ri == 1) && (ci == 0))
    k = get_kgw(pw,pg,ipp);
  if((ri == 1) && (ci == 1))
    k = get_kgg(pw,pg,ipp);
  
  fillm(0.0,d);
  
  d[0][0]=k;   d[0][1]=0.0; d[0][2]=0.0;
  d[1][0]=0.0; d[1][1]=k;   d[1][2]=0.0;
  d[2][0]=0.0; d[2][1]=0.0; d[2][2]=k;
}


/**
   function creates capacity %matrix of the material

   @param c   - capacity %matrix of the material
   @param ri  - row index
   @param ci  - column index   
   @param ipp - number of integration point
*/
void con_awf2mat::matcap (double &c,long ri,long ci,long ipp)
{
  double pw,pg;
  c = 0.0;
  
  pw = Tm->ip[ipp].av[0];
  pg = Tm->ip[ipp].av[1];
  
  if((ri == 0) && (ci == 0))
    c = get_capww(pw,pg,ipp);
  if((ri == 0) && (ci == 1))
    c = get_capwg(pw,pg,ipp);
  if((ri == 1) && (ci == 0))
    c = get_capgw(pw,pg,ipp);
  if((ri == 1) && (ci == 1))
    c = get_capgg(pw,pg,ipp);
}


/**
   function computes volume part of right-hand side matrix
   in the required integration point
   
   @param d - right-hand side %matrix of material
   @param ri  - row index
   @param ci  - column index   
   @param ipp - number of integration point
*/
void con_awf2mat::rhs_volume (matrix &d,long ri,long ci,long ipp)
{
  long m;
  m = d.m;
  
  switch (m){
  case 1:{
    rhs1d1 (d,ri,ci,ipp);//1D
    break;
  }
  case 2:{
    rhs2d1 (d,ri,ci,ipp);//2D
    break;
  }
  case 3:{
    rhs3d1 (d,ri,ci,ipp);//3D
    break;
  }
  default:{
    fprintf (stderr,"\n unknown number of components of stress tensor is required");
    fprintf (stderr,"\n in function (%s, line %d).\n",__FILE__,__LINE__);
  }
  }
}



/**
   function creates volume right-hand side matrix of the material for 1D problems
   @param d - right-hand %matrix of the material
   @param ri  - row index
   @param ci  - column index   
   @param ipp - number of integration point
*/
void con_awf2mat::rhs1d1 (matrix &d,long ri,long ci,long ipp)
{
  double f,pw,pg;
  double g;
  
  pw = Tm->ip[ipp].av[0];
  pg = Tm->ip[ipp].av[1];
  
  if(ri == 0){
    f = get_fw1(pw,pg,ipp);
    g = Tp->gr1;//vector of gravity acceleration
    
    fillm(0.0,d);
    d[0][0] = f*g;
  }
  if(ri == 1){
    f = get_fg1(pw,pg,ipp);
    g = Tp->gr1;//vector of gravity acceleration

    fillm(0.0,d);
    d[0][0] = f*g;
  }
}

/**
   function creates volume right-hand side matrix of the material for 2D problems
   @param d - right-hand %matrix of the material
   @param ri  - row index
   @param ci  - column index   
   @param ipp - number of integration point
*/
void con_awf2mat::rhs2d1 (matrix &d,long ri,long ci,long ipp)
{
  double f,pw,pg;
  double *g;
  
  pw = Tm->ip[ipp].av[0];
  pg = Tm->ip[ipp].av[1];
  
  if(ri == 0){
    g = new double [2];
    f = get_fw1(pw,pg,ipp);

    g[0] = Tp->gr1;//gravity acceleration
    g[1] = Tp->gr2;//gravity acceleration
    
    fillm(0.0,d);
    d[0][0] = f*g[0];
    d[1][0] = f*g[1];
  }
  if(ri == 1){
    g = new double [2];
    f = get_fg1(pw,pg,ipp);

    g[0] = Tp->gr1;//gravity acceleration
    g[1] = Tp->gr2;//gravity acceleration

    fillm(0.0,d);
    d[0][0] = f*g[0];
    d[1][0] = f*g[1];
  }
}

/**
   function creates volume right-hand side matrix of the material for 3D problems
   @param d - right-hand %matrix of the material
   @param ri  - row index
   @param ci  - column index   
   @param ipp - number of integration point
*/
void con_awf2mat::rhs3d1 (matrix &d,long ri,long ci,long ipp)
{
  double f,pw,pg;
  double *g;
  
  pw = Tm->ip[ipp].av[0];
  pg = Tm->ip[ipp].av[1];
  
  if(ri == 0){
    g = new double [3];
    f = get_fw1(pw,pg,ipp);

    g[0] = Tp->gr1;//gravity acceleration
    g[1] = Tp->gr2;//gravity acceleration
    g[2] = Tp->gr3;//gravity acceleration

    fillm(0.0,d);
    d[0][0] = f*g[0];
    d[1][0] = f*g[1];
    d[2][0] = f*g[2];
  }
  if(ri == 1){
    g = new double [3];
    f = get_fg1(pw,pg,ipp);

    g[0] = Tp->gr1;//gravity acceleration
    g[1] = Tp->gr2;//gravity acceleration
    g[2] = Tp->gr3;//gravity acceleration

    fillm(0.0,d);
    d[0][0] = f*g[0];
    d[1][0] = f*g[1];
    d[2][0] = f*g[2];
  }
}


/**
   function computes degree of saturation(sorption curve)
   @param pw - water pressure
   @param pg - air pressure

   @retval sw - degree of saturation

   29/10/2009, TKr
*/
double con_awf2mat::get_sw(double pw, double pg, long ipp)
{
  double sw,pc;
  
  //pc = pg - pw;
  pc = -pw;//debug
  
  switch (model_type){
  case lewis_and_schrefler2:{//Lewis and Schrefler's book
    sw = 1.0 - (1.9722e-11)*pow(pc,2.4279);//p. 167
  }
  case lewis_and_schrefler2_mefel:{//saturation degree ind its derivative are obtained from mefel;
    if(Tm->nontransq != NULL){
      sw = Tm->givenontransq(saturation_deg, ipp); //actual saturation degree
    }
    break;
  }
  default:{
    fprintf (stderr,"\n unknown model type is required");
    fprintf (stderr,"\n in function (%s, line %d).\n",__FILE__,__LINE__);
  }
  }  
  
  
  //fprintf (Outt,"ipp = %ld; sw = %e\n",ipp,sw);
  //fflush(Outt);

  return(sw);
}



/**
   function computes specific moisture content (partial derivative of degree of saturation with respect to pc)
   @param pw - water pressure
   @param pg - air pressure

   @retval cs - specific moisture content = partial derivative of degree of saturation with respect to pc


   29/10/2009, TKr
*/
double con_awf2mat::get_cs(double pw, double pg, long ipp)
{
  double cs,pc;
  
  //pc = pg - pw;
  pc = -pw;//debug
  
  switch (model_type){
  case lewis_and_schrefler2:{//Lewis and Schrefler's book
    cs = -2.4279*(1.9722e-11)*pow(pc,(2.4279-1.0));//p. 167
    break;
  }
  case lewis_and_schrefler2_mefel:{//saturation degree ind its derivative are obtained from mefel;
    if(Tm->nontransq != NULL){
      cs = -Tm->givenontransq(der_saturation_deg, ipp); //actual derivative of saturation degree
    }
    break;
  }
  default:{
    fprintf (stderr,"\n unknown model type is required");
    fprintf (stderr,"\n in function (%s, line %d).\n",__FILE__,__LINE__);
  }
  } 

  //fprintf (Outt,"ipp = %ld; dsw_dpw = %e\n",ipp,-cs);
  //fflush(Outt);

  return(cs);
}



/**
   function computes degree of saturation se (internal material variable)
   @param pw - water pressure
   @param pg - air pressure

   @retval se - degree of saturation se

   29/10/2009, TKr
*/
double con_awf2mat::get_se(double sw)
{
  double se;
  
  se = (sw - 0.2)*(1.0 - 0.2);//p. 169

  return(se);
}




/**
   function computes water relative permeability
   @param pw - water pressure
   @param pg - air pressure
   
   @retval krw - water relative permeability

   12/9/2008, TKr
*/
double con_awf2mat::get_krw(double pw, double pg, long ipp)
{
  double sw,krw;
  
  krw=1.0;

  switch (model_type){
  case lewis_and_schrefler:{//Lewis and Schrefler's book
    //sw = get_sw(pw,pg,ipp);//docasne zakomentovano??!!
    //krw = 1.0 + 2207*pow((1.0-sw),1.0121);//p. 167
    break;
  }
  case lewis_and_schrefler_mefel:{//Lewis and Schrefler's book, saturation degree ind its derivative are obtained from mefel;
    //s = get_sw(pw,ipp);//docasne zakomentovano??!!
    //this formula is teporarily here??!!
    //krw = 1.0 + 2207*pow((1.0-sw),1.0121);//p. 167
    break;
  }
  default:{
    fprintf (stderr,"\n unknown model type is required");
    fprintf (stderr,"\n in function (%s, line %d).\n",__FILE__,__LINE__);
  }
  }

  return(krw);
}



/**
   function computes air relative permeability
   @param pw - water pressure
   @param pg - air pressure
   
   @retval krg - gas relative permeability

   12/9/2008, TKr
*/
double con_awf2mat::get_krg(double pw, double pg, long ipp)
{
  double krg,sw,se;

  krg=1.0;

  switch (model_type){
  case lewis_and_schrefler:{//Lewis and Schrefler's book
    //sw = get_sw(pw,pg,ipp);//docasne zakomentovano??!!
    //se = get_se(sw);
    //krg = pow((1.0-se),2.0)*(1.0 - pow(se,(5.0/3.0)));//p. 169
    
    //if((krg < 0.0001) || (sw >= 0.998))
    //krg = 0.0001;
    break;
  }
  case lewis_and_schrefler_mefel:{//Lewis and Schrefler's book, saturation degree ind its derivative are obtained from mefel;
    //sw = get_sw(pw,pg,ipp);//docasne zakomentovano??!!
    //se = get_se(sw);
    //krg = pow((1.0-se),2.0)*(1.0 - pow(se,(5.0/3.0)));//p. 169
    
    //if((krg < 0.0001) || (sw >= 0.998))
    //krg = 0.0001;
    break;
  }
  default:{
    fprintf (stderr,"\n unknown model type is required");
    fprintf (stderr,"\n in function (%s, line %d).\n",__FILE__,__LINE__);
  }
  }

  return(krg);
}



/**
   function creates conductivity coefficient of the general material
   @param pw - water pressure
   @param pg - air pressure

   @retval kww - conductivity coefficient

   29/10/2009, TKr
*/
double con_awf2mat::get_kww(double pw, double pg, long ipp)
{
  double krw,kww;
  
  krw = get_krw(pw,pg,ipp);
  kww = krw*kintr/muw0;//p. 97

  //fprintf (Outt,"ipp = %ld; kww = %e\n",ipp,kww);
  //fflush(Outt);

  return(kww);
}


/**
   function creates conductivity coefficient of the general material
   @param pw - water pressure
   @param pg - air pressure

   @retval kwg - conductivity coefficient

   29/10/2009, TKr
*/
double con_awf2mat::get_kwg(double pw, double pg, long ipp)
{
  double kwg;
  
  kwg = 0.0;

  return(kwg);

}


/**
   function creates conductivity coefficient of the general material
   @param pw - water pressure
   @param pg - air pressure

   @retval kgw - conductivity coefficient

   29/10/2009, TKr
*/
double con_awf2mat::get_kgw(double pw, double pg, long ipp)
{
  double kgw;
  
  kgw = 0.0;

  return(kgw);

}


/**
   function creates conductivity coefficient of the general material
   @param pw - water pressure
   @param pg - air pressure

   @retval kgg - conductivity coefficient

   29/10/2009, TKr
*/
double con_awf2mat::get_kgg(double pw, double pg, long ipp)
{
  double krg,kgg;
  
  krg = get_krg(pw,pg,ipp);
  kgg = krg*kintr/mug0;//p. 97

  return(kgg);
}



/**
   function creates capacity coefficient of the general material
   @param pw - water pressure
   @param pg - air pressure

   @retval capww - capacity coefficient

   29/10/2009, TKr
*/
double con_awf2mat::get_capww(double pw, double pg, long ipp)
{
  double sw,cs,n,capww;  
  
  sw = get_sw(pw,pg,ipp);
  cs = get_cs(pw,pg,ipp);
  n = phi0;
  
  //capww = (alpha-n)/ks*sw*(sw - pw*cs/n + pg*cs/n) + n*sw/kw - cs;//p. 96//this is wrong
  capww = (alpha-n)/ks*sw*(sw - pw*cs + pg*cs) + n*sw/kw - n*cs;//this equation is correct

  //fprintf (Outt,"ipp = %ld; cappw = %e\n",ipp,capww);
  //fflush(Outt);

  return(capww);

}


/**
   function creates capacity coefficient of the general material
   @param pw - water pressure
   @param pg - air pressure

   @retval capwg - capacity coefficient

   29/10/2009, TKr
*/
double con_awf2mat::get_capwg(double pw, double pg, long ipp)
{
  double sw,sg,cs,n,capwg;  
  
  sw = get_sw(pw,pg,ipp);
  sg = 1.0 - sw;
  cs = get_cs(pw,pg,ipp);
  n = phi0;
  
  //capwg = (alpha-n)/ks*sw*(sg + pw*cs/n - pg*cs/n) + cs;//p. 96 this equation is wrong
  capwg = (alpha-n)/ks*sw*(sg + pw*cs - pg*cs) + n*cs;//this equation is correct

  return(capwg);

}


/**
   function creates capacity coefficient of the general material
   @param pw - water pressure
   @param pg - air pressure

   @retval capgw - capacity coefficient

   29/10/2009, TKr
*/
double con_awf2mat::get_capgw(double pw, double pg, long ipp)
{
  double sw,sg,cs,n,capgw;  
  
  sw = get_sw(pw,pg,ipp);
  sg = 1.0 - sw;
  cs = get_cs(pw,pg,ipp);
  n = phi0;
  
  //capgw = (alpha-n)/ks*sg*(sw + pg*cs/n - pw*cs/n) + cs;//p. 97 this equation is wrong
  capgw = (alpha-n)/ks*sg*(sw + pg*cs - pw*cs) + n*cs;//this equation is correct

  return(capgw);
}


/**
   function creates capacity coefficient of the general material
   @param pw - water pressure
   @param pg - air pressure

   @retval capgg - capacity coefficient

   29/10/2009, TKr
*/
double con_awf2mat::get_capgg(double pw, double pg, long ipp)
{
  double sw,sg,cs,n,capgg;  
  
  sw = get_sw(pw,pg,ipp);
  sg = 1.0 - sw;
  cs = get_cs(pw,pg,ipp);
  n = phi0;

  //capgg = (alpha-n)/ks*sg*(sg - pg*cs/n + pw*cs/n) + n*sg/pg - cs;//p. 97//this is wrong
  capgg = (alpha-n)/ks*sg*(sg - pg*cs + pw*cs) + n*sg/pg - n*cs;//this equation is correct

  return(capgg);

}


/**
   function returns coefficient for righ-hand side of the general material
   @param pw - water pressure
   @param pg - air pressure

   @retval fw - first part for right-hand side coeficient for continutiy equation

   29/10/2009, TKr
*/
double con_awf2mat::get_fw1(double pw, double pg, long ipp)
{
  double fw1,krw;

  krw = get_krw(pw,pg,ipp);
  
  fw1 = krw*kintr/muw0*rhow;//p. 97

  return(fw1);

}


/**
   function returns coefficient for righ-hand side of the general material 
   @param pw - water pressure
   @param pg - air pressure

   @retval fg - first part for right-hand side for continutiy equation

   29/10/2009, TKr
*/
double con_awf2mat::get_fg1(double pw, double pg, long ipp)
{
  double fg1,krg,rhog;

  krg = get_krg(pw,pg,ipp);
  rhog = 1.25;//temporarilly

  fg1 = krg*kintr/mug0*rhog;//p. 97

  return(fg1);

}


/**
   function computes all other variables at nodes
   @param compother - number of other components
   @param pw - water capillary pressure on actual node
   @param pg - gas(air) pressure on actual node
   @param ipp - first integration point on element

   @retval other - other variable

   03/03/2011, TKr
*/

double con_awf2mat::get_othervalue(long compother,double pw, double pg, long ipp)
{
  double other;
  state_eq tt;

  switch (compother){
  case 0:{//capillary pressure
    other = pg - pw;
      break;
  }
  case 1:{//gas pressure
    other = pg;
    break;
  }
  case 2:{//saturation
    other = get_sw(pw,pg,ipp);
    break;
  }
  case 3:{//liquid water pressure
    other = pw;
    break;
  }
  case 4:{//cpecific moisture content
    other = get_cs(pw,pg,ipp);//temporarilly
    break;
  }    
  default:{
    fprintf (stderr,"\n\n unknown type of component is required in function (%s, line %d).\n",__FILE__,__LINE__);
  }
  }
  return (other);

}



/**
     function prints names of all other variables at nodes
     @param out - output rhle
     @param compother - number of other components

     03/03/2011, TKr
*/
void con_awf2mat::print_othervalue_name(FILE *out,long compother)
{
  switch (compother){
  case 0:{//capillary pressure
    fprintf (out,"Capillary pressure (Pa)");
    break;
  }
  case 1:{//gas pressure
    fprintf (out,"Gas pressure (Pa)             ");
    break;
  }
  case 2:{//saturation
    fprintf (out,"Degree of saturation ()       ");
    break;
  }
  case 3:{//liquid water pressure
    fprintf (out,"Pore water pressure (Pa)      ");
    break;
  }
  case 4:{//moisture content
    fprintf (out,"Moisture content (kg/kg)      ");
    break;
  }    
  default:{
    fprintf (stderr,"\n\n unknown type of component is required in function (%s, line %d).\n",__FILE__,__LINE__);
  }
  }
}



/**
   function checks if computed unknowns are physically reasonable
   @param nv  - vector of unknowns
   @param ipp - number of integration point


   03/03/2011, TKr
*/
void con_awf2mat::values_correction (vector &nv, long ipp)
{
  //  pore water pressure control
  water_pressure_check(nv[0],nv[1],ipp);
}


/**
   function checks if water pressure is non-positive
   @param pw - pore water pressure
   @param ipp - number of integration point


   03/03/2011, TKr
*/
void con_awf2mat::water_pressure_check(double &pw,double &pg,long ipp)
{
  if (pw >= 0.0) 
    pw = -1.0;
  
  if (pg <= 0.0)
    pg = 1.0;

  //storing into gauss point
  Tm->ip[ipp].av[0] = pw;
  Tm->ip[ipp].av[1] = pg;
}

/**
  This function updates values in the other array reached in the previous equlibrium state to
  values reached in the new actual equilibrium state.

  @param ipp - integration point number in the mechmat ip array.

   23/05/2016, TKr
*/
void con_awf2mat::updateval (long ipp)
{
}


/**
   This function initializes material model data
   with respect of consistency parameter gamma.

   @param ipp - integration point number

   23/05/2016, TKr
*/
void con_awf2mat::initval(long ipp)
{
}



/**
  The function returns ordered dof names of primary unknowns 
  required by the model.

  @param dofname   - array of uknown name for particular nodal dofs (output)
                     dofname[i] = name of i-th nodal unknown (for names see aliast.h - enum namevart)
  @param ntm       - number of transported media = number of nodal dof = length of array dofname

  Created by Tomas Krejci according to Tomas Koudelka, 23/05/2016
*/
void con_awf2mat::give_dof_names(namevart *dofname, long ntm)
{
  if (ntm < 1)
  {
    print_err("the model defines %ld unknowns but number of transported media is %ld", 
              __FILE__, __LINE__, __func__, 1, ntm);
    abort();
  }
  dofname[0] = trf_water_press;
  dofname[1] = trf_gas_press;
}


/**
   function returns water pressure

   @param ipp - integration point number

   @retval pw - pore water pressure

   27/05/2016, TKr
*/
double con_awf2mat::give_water_pressure(long ipp)
{
  double pw;

  pw = Tm->ip[ipp].av[0];

  return(pw);
}

/**
   function returns pore pressure

   @param ipp - integration point number

   @retval pp - pore pressure

   27/05/2016, TKr
*/
double con_awf2mat::give_pore_pressure(long ipp)
{
  double pp,pg,pw;

  pw = Tm->ip[ipp].av[0];
  pg = Tm->ip[ipp].av[1];

  pp = pg - pw;

  return(pp);
}

/**
   function returns gas pressure

   @param ipp - integration point number

   @retval pg - pore gas pressure

   27/05/2016, TKr
*/
double con_awf2mat::give_gas_pressure(long ipp)
{
  double pg;

  pg = Tm->ip[ipp].av[1];

  return(pg);
}


/**
   function computes suction stress s = -(pg - pw) = -pc;
   @param ipp - integration point number

   @retval suction - suction stress [Pa]

   27/05/2016, TKr
*/
double con_awf2mat::give_suction(long ipp)
{
  double pw,pg,suction;
  
  pw = Tm->ip[ipp].av[0];
  pg = Tm->ip[ipp].av[1];
  
  switch (model_type){
  case lewis_and_schrefler2:{//Lewis and Schrefler's book
    suction = -1.0*(pg - pw);//this is correct, because capillary water pressure is negative
    break;
  }
  case lewis_and_schrefler2_mefel:{//Lewis and Schrefler's book, saturation degree ind its derivative are obtained from mefel
    pg = pg - p_atm;//only suction is needed
    suction = -1.0*(pg - pw);//this is correct, because capillary water pressure is negative
    //suction = pw;//debug??!!
    break;
  }
  default:{
    fprintf (stderr,"\n unknown model type is required");
    fprintf (stderr,"\n in function (%s, line %d).\n",__FILE__,__LINE__);
  }
  } 
  return(suction);
}


/**
   function computes suction stress (s = -pw = pc);
   @param ipp - integration point number

   @retval suction - suction stress [Pa]

   27/05/2016, TKr
*/
double con_awf2mat::give_saturation_degree(long ipp)
{
  double pw,pg,s;
  
  pw = Tm->ip[ipp].av[0];
  pg = Tm->ip[ipp].av[1];

  s = get_sw(pw,pg,ipp);
  
  return(s);
}

/**
  The funtion marks required non-transport quantities in the array antq.

  @param antq - array with flags for used material types
                antq[i] = 1 => quantity type nontransquant(i+1) is required
                antq[i] = 0 => quantity type nontransquant(i+1) is not required

  @return The function does not return anything, but it may change content of antq array.
  
  27/05/2016, TKr
*/
void con_awf2mat::give_reqntq(long *antq)
{
  switch (model_type){
  case lewis_and_schrefler2:{//Lewis and Schrefler's book
    break;
  }
  case lewis_and_schrefler2_mefel:{//Lewis and Schrefler's book, saturation degree ind its derivative are obtained from mefel;
    antq[saturation_deg-1] = 1;
    antq[der_saturation_deg-1] = 1;
    break;
  }  
  default:{
    print_err("unknown model type is required", __FILE__, __LINE__, __func__);
  }
  } 
}
